/***************************************************************************

  Caveman Ninja     (c) 1991 Data East Corporation (World version)
  Caveman Ninja     (c) 1991 Data East Corporation (USA version)
  Joe & Mac         (c) 1991 Data East Corporation (Japanese version)
  Stone Age         (Italian bootleg)

  The 'USA' set has a 'Winners Don't Use Drugs' screen, the other set doesn't.

Sound (Original):
  Hu6280 CPU
  YM2151 (music)
  YM2203C (effects)
  2 Oki ADPCM chips

Sound (Bootleg):
  Z80 (Program ripped from Block Out!)
  YM2151
  Oki chip

Issues:
  Colours can be wrong when gfx banks are switching.
  End of level 2 is corrupt.

  Emulation by Bryan McPhail, mish@tendril.force9.net

***************************************************************************/

#include "driver.h"
#include "vidhrdw/generic.h"
#include "h6280/h6280.h"
#include "z80/z80.h"

/* Video emulation definitions */
extern unsigned char *cninja_pf1_rowscroll,*cninja_pf2_rowscroll;
extern unsigned char *cninja_pf3_rowscroll,*cninja_pf4_rowscroll;
extern unsigned char *cninja_pf1_data,*cninja_pf2_data;
extern unsigned char *cninja_pf3_data,*cninja_pf4_data;

int  cninja_vh_start(void);
void cninja_vh_stop(void);
void cninja_vh_screenrefresh(struct osd_bitmap *bitmap,int full_refresh);

int cninja_pf1_data_r(int offset);

void cninja_pf1_data_w(int offset,int data);
void cninja_pf2_data_w(int offset,int data);
void cninja_pf3_data_w(int offset,int data);
void cninja_pf4_data_w(int offset,int data);
void cninja_control_0_w(int offset,int data);
void cninja_control_1_w(int offset,int data);

void cninja_pf1_rowscroll_w(int offset,int data);
void cninja_pf2_rowscroll_w(int offset,int data);
void cninja_pf3_rowscroll_w(int offset,int data);
void cninja_pf4_rowscroll_w(int offset,int data);

int cninja_pf3_rowscroll_r(int offset);

void cninja_palette_24bit_w(int offset,int data);

static int loopback[0x100];
static unsigned char *cninja_ram;

/**********************************************************************************/

static void cninja_sound_w(int offset, int data)
{
	soundlatch_w(0,data&0xff);
	cpu_cause_interrupt(1,H6280_INT_IRQ1);
}

static void stoneage_sound_w(int offset, int data)
{
	soundlatch_w(0,data&0xff);
	cpu_cause_interrupt(1,Z80_NMI_INT);
}

static void cninja_loopback_w(int offset, int data)
{
	WRITE_WORD(&loopback[offset],data);
}

static int cninja_prot_r(int offset)
{
 	switch (offset) {
		case 0x80: /* Master level control */
			return READ_WORD(&loopback[0]);

		case 0xde: /* Restart position control */
			return READ_WORD(&loopback[2]);

		case 0xe6: /* The number of credits in the system. */
			return READ_WORD(&loopback[4]);

		case 0x86: /* End of game check.  See 0x1814 */
			return READ_WORD(&loopback[6]);

		/* Video registers */
		case 0x5a: /* Moved to 0x140000 on int */
			return READ_WORD(&loopback[0x10]);
		case 0x84: /* Moved to 0x14000a on int */
			return READ_WORD(&loopback[0x12]);
		case 0x20: /* Moved to 0x14000c on int */
			return READ_WORD(&loopback[0x14]);
		case 0x72: /* Moved to 0x14000e on int */
			return READ_WORD(&loopback[0x16]);
		case 0xdc: /* Moved to 0x150000 on int */
			return READ_WORD(&loopback[0x18]);
		case 0x6e: /* Moved to 0x15000a on int */
			return READ_WORD(&loopback[0x1a]); /* Not used on bootleg */
		case 0x6c: /* Moved to 0x15000c on int */
			return READ_WORD(&loopback[0x1c]);
		case 0x08: /* Moved to 0x15000e on int */
			return READ_WORD(&loopback[0x1e]);

		case 0x36: /* Dip switches */
			return (readinputport(3) + (readinputport(4) << 8));

		case 0x1c8: /* Coins */
			return readinputport(2);

		case 0x22c: /* Player 1 & 2 input ports */
			return (readinputport(0) + (readinputport(1) << 8));
	}
//	if (errorlog) fprintf(errorlog,"Protection PC %06x: warning - read unmapped memory address %04x\n",cpu_getpc(),offset);
	return 0;
}

/**********************************************************************************/

static struct MemoryReadAddress cninja_readmem[] =
{
	{ 0x000000, 0x0bffff, MRA_ROM },
	{ 0x144000, 0x144fff, cninja_pf1_data_r },
	{ 0x15c000, 0x15c7ff, cninja_pf3_rowscroll_r },
	{ 0x184000, 0x187fff, MRA_BANK1 },
	{ 0x190004, 0x190005, MRA_NOP }, /* Seemingly unused */
	{ 0x19c000, 0x19dfff, paletteram_word_r },
	{ 0x1a4000, 0x1a47ff, MRA_BANK2 }, /* Sprites */
	{ 0x1bc000, 0x1bcfff, cninja_prot_r }, /* Protection device */
	{ -1 }  /* end of table */
};

static struct MemoryWriteAddress cninja_writemem[] =
{
	{ 0x000000, 0x0bffff, MWA_ROM },

	{ 0x140000, 0x14000f, cninja_control_1_w },
	{ 0x144000, 0x144fff, cninja_pf1_data_w, &cninja_pf1_data },
	{ 0x146000, 0x146fff, cninja_pf4_data_w, &cninja_pf4_data },
	{ 0x14c000, 0x14c7ff, cninja_pf1_rowscroll_w, &cninja_pf1_rowscroll },
	{ 0x14e000, 0x14e7ff, cninja_pf4_rowscroll_w, &cninja_pf4_rowscroll },

	{ 0x150000, 0x15000f, cninja_control_0_w },
	{ 0x154000, 0x154fff, cninja_pf3_data_w, &cninja_pf3_data },
	{ 0x156000, 0x156fff, cninja_pf2_data_w, &cninja_pf2_data },
	{ 0x15c000, 0x15c7ff, cninja_pf3_rowscroll_w, &cninja_pf3_rowscroll },
	{ 0x15e000, 0x15e7ff, cninja_pf2_rowscroll_w, &cninja_pf2_rowscroll },

	{ 0x184000, 0x187fff, MWA_BANK1, &cninja_ram }, /* Main ram */
	{ 0x190000, 0x190007, MWA_NOP }, /* IRQ Ack + DMA flags? */
	{ 0x19c000, 0x19dfff, cninja_palette_24bit_w, &paletteram },
	{ 0x1a4000, 0x1a47ff, MWA_BANK2, &spriteram },
	{ 0x1b4000, 0x1b4001, MWA_NOP }, /* DMA flag to graphics chips? */
	{ 0x1bc000, 0x1bc0ff, cninja_loopback_w }, /* Protection writes */

	{ 0x308000, 0x308fff, MWA_NOP }, /* Bootleg only */

	{ -1 }  /* end of table */
};

/******************************************************************************/

static void YM2151_w(int offset, int data)
{
	switch (offset) {
	case 0:
		YM2151_register_port_0_w(0,data);
		break;
	case 1:
		YM2151_data_port_0_w(0,data);
		break;
	}
}

static void YM2203_w(int offset, int data)
{
	switch (offset) {
	case 0:
		YM2203_control_port_0_w(0,data);
		break;
	case 1:
		YM2203_write_port_0_w(0,data);
		break;
	}
}

/* Physical memory map (21 bits) */
static struct MemoryReadAddress sound_readmem[] =
{
	{ 0x000000, 0x00ffff, MRA_ROM },
	{ 0x100000, 0x100001, YM2203_status_port_0_r },
	{ 0x110000, 0x110001, YM2151_status_port_0_r },
	{ 0x120000, 0x120001, OKIM6295_status_0_r },
	{ 0x130000, 0x130001, OKIM6295_status_1_r },
	{ 0x140000, 0x140001, soundlatch_r },
	{ 0x1f0000, 0x1f1fff, MRA_BANK8 },
	{ -1 }  /* end of table */
};

static struct MemoryWriteAddress sound_writemem[] =
{
	{ 0x000000, 0x00ffff, MWA_ROM },
	{ 0x100000, 0x100001, YM2203_w },
	{ 0x110000, 0x110001, YM2151_w },
	{ 0x120000, 0x120001, OKIM6295_data_0_w }, /* Effects  */
	{ 0x130000, 0x130001, OKIM6295_data_1_w }, /* Music samples */
	{ 0x1f0000, 0x1f1fff, MWA_BANK8 },
	{ 0x1fec00, 0x1fec01, H6280_timer_w },
	{ 0x1ff402, 0x1ff403, H6280_irq_status_w },
    { -1 }  /* end of table */
};

static struct MemoryReadAddress stoneage_s_readmem[] =
{
	{ 0x0000, 0x7fff, MRA_ROM },
	{ 0x8000, 0x87ff, MRA_RAM },
	{ 0x8801, 0x8801, YM2151_status_port_0_r },
	{ 0xa000, 0xa000, soundlatch_r },
	{ 0x9800, 0x9800, OKIM6295_status_0_r },
	{ -1 }  /* end of table */
};

static struct MemoryWriteAddress stoneage_s_writemem[] =
{
	{ 0x0000, 0x7fff, MWA_ROM },
	{ 0x8000, 0x87ff, MWA_RAM },
	{ 0x8800, 0x8800, YM2151_register_port_0_w },
	{ 0x8801, 0x8801, YM2151_data_port_0_w },
	{ 0x9800, 0x9800, OKIM6295_data_0_w },
	{ -1 }  /* end of table */
};

/**********************************************************************************/

#define PORTS_COINS \
	PORT_START \
	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_8WAY ) \
	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_8WAY ) \
	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_8WAY ) \
	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_8WAY ) \
	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON1 ) \
	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_BUTTON2 ) \
	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_UNUSED ) \
	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_START1 ) \
	PORT_START \
	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_UP    | IPF_8WAY | IPF_COCKTAIL ) \
	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN  | IPF_8WAY | IPF_COCKTAIL ) \
	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT  | IPF_8WAY | IPF_COCKTAIL ) \
	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_8WAY | IPF_COCKTAIL ) \
	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_COCKTAIL ) \
	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_BUTTON2 | IPF_COCKTAIL ) \
	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_UNUSED ) \
	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_START2 ) \
	PORT_START \
	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_COIN1 ) \
	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_COIN2 ) \
	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_COIN3 ) \
	PORT_BIT( 0x08, IP_ACTIVE_HIGH, IPT_VBLANK ) \
	PORT_BIT( 0xf0, IP_ACTIVE_LOW, IPT_UNKNOWN )


INPUT_PORTS_START( input_ports )

PORTS_COINS

	PORT_START	/* Dip switch bank 1 */
	PORT_DIPNAME( 0x07, 0x07, "Coin A", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "3 Coins/1 Credit" )
	PORT_DIPSETTING(    0x01, "2 Coins/1 Credit" )
	PORT_DIPSETTING(    0x07, "1 Coin/1 Credit" )
	PORT_DIPSETTING(    0x06, "1 Coin/2 Credits" )
	PORT_DIPSETTING(    0x05, "1 Coin/3 Credits" )
	PORT_DIPSETTING(    0x04, "1 Coin/4 Credits" )
	PORT_DIPSETTING(    0x03, "1 Coin/5 Credits" )
	PORT_DIPSETTING(    0x02, "1 Coin/6 Credits" )
	PORT_DIPNAME( 0x38, 0x38, "Coin B", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "3 Coins/1 Credit" )
	PORT_DIPSETTING(    0x08, "2 Coins/1 Credit" )
	PORT_DIPSETTING(    0x38, "1 Coin/1 Credit" )
	PORT_DIPSETTING(    0x30, "1 Coin/2 Credits" )
	PORT_DIPSETTING(    0x28, "1 Coin/3 Credits" )
	PORT_DIPSETTING(    0x20, "1 Coin/4 Credits" )
	PORT_DIPSETTING(    0x18, "1 Coin/5 Credits" )
	PORT_DIPSETTING(    0x10, "1 Coin/6 Credits" )
	PORT_DIPNAME( 0x40, 0x40, "Flip_Screen", IP_KEY_NONE )
	PORT_DIPSETTING(    0x40, "Off" )
	PORT_DIPSETTING(    0x00, "On" )
	PORT_DIPNAME( 0x80, 0x80, "Unknown", IP_KEY_NONE )
	PORT_DIPSETTING(    0x80, "Off" )
	PORT_DIPSETTING(    0x00, "On" )

	PORT_START	/* Dip switch bank 2 */
	PORT_DIPNAME( 0x03, 0x03, "Lives", IP_KEY_NONE )
	PORT_DIPSETTING(    0x01, "1" )
	PORT_DIPSETTING(    0x00, "2" )
	PORT_DIPSETTING(    0x03, "3" )
	PORT_DIPSETTING(    0x02, "4" )
	PORT_DIPNAME( 0x0c, 0x0c, "Difficulty", IP_KEY_NONE )
	PORT_DIPSETTING(    0x08, "Easy" )
	PORT_DIPSETTING(    0x0c, "Normal" )
	PORT_DIPSETTING(    0x04, "Hard" )
	PORT_DIPSETTING(    0x00, "Hardest" )
	PORT_DIPNAME( 0x10, 0x10, "Unknown", IP_KEY_NONE )
	PORT_DIPSETTING(    0x10, "Off" )
	PORT_DIPSETTING(    0x00, "On" )
	PORT_DIPNAME( 0x20, 0x20, "Unknown", IP_KEY_NONE )
	PORT_DIPSETTING(    0x20, "Off" )
	PORT_DIPSETTING(    0x00, "On" )
	PORT_DIPNAME( 0x40, 0x40, "Unknown", IP_KEY_NONE )
	PORT_DIPSETTING(    0x40, "Off" )
	PORT_DIPSETTING(    0x00, "On" )
	PORT_DIPNAME( 0x80, 0x80, "Demo Sounds", IP_KEY_NONE )
	PORT_DIPSETTING(    0x80, "Off" )
	PORT_DIPSETTING(    0x00, "On" )
INPUT_PORTS_END


INPUT_PORTS_START( cninjau_input_ports )

PORTS_COINS

	PORT_START	/* Dip switch bank 1 */
	PORT_DIPNAME( 0x07, 0x07, "Coin A", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "3 Coins/1 Credit" )
	PORT_DIPSETTING(    0x01, "2 Coins/1 Credit" )
	PORT_DIPSETTING(    0x07, "1 Coin/1 Credit" )
	PORT_DIPSETTING(    0x06, "1 Coin/2 Credits" )
	PORT_DIPSETTING(    0x05, "1 Coin/3 Credits" )
	PORT_DIPSETTING(    0x04, "1 Coin/4 Credits" )
	PORT_DIPSETTING(    0x03, "1 Coin/5 Credits" )
	PORT_DIPSETTING(    0x02, "1 Coin/6 Credits" )
	PORT_DIPNAME( 0x38, 0x38, "Coin B", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "3 Coins/1 Credit" )
	PORT_DIPSETTING(    0x08, "2 Coins/1 Credit" )
	PORT_DIPSETTING(    0x38, "1 Coin/1 Credit" )
	PORT_DIPSETTING(    0x30, "1 Coin/2 Credits" )
	PORT_DIPSETTING(    0x28, "1 Coin/3 Credits" )
	PORT_DIPSETTING(    0x20, "1 Coin/4 Credits" )
	PORT_DIPSETTING(    0x18, "1 Coin/5 Credits" )
	PORT_DIPSETTING(    0x10, "1 Coin/6 Credits" )
	PORT_DIPNAME( 0x40, 0x40, "Flip Screen", IP_KEY_NONE )
	PORT_DIPSETTING(    0x40, "Off" )
	PORT_DIPSETTING(    0x00, "On" )
	PORT_DIPNAME( 0x80, 0x80, "Credit(s) to Start", IP_KEY_NONE )
	PORT_DIPSETTING(    0x80, "1" )
	PORT_DIPSETTING(    0x00, "2" )
/* Also, if Coin A and B are on 1/1, 0x00 gives 2 to start, 1 to continue */

	PORT_START	/* Dip switch bank 2 */
	PORT_DIPNAME( 0x03, 0x03, "Lives", IP_KEY_NONE )
	PORT_DIPSETTING(    0x01, "1" )
	PORT_DIPSETTING(    0x00, "2" )
	PORT_DIPSETTING(    0x03, "3" )
	PORT_DIPSETTING(    0x02, "4" )
	PORT_DIPNAME( 0x0c, 0x0c, "Difficulty", IP_KEY_NONE )
	PORT_DIPSETTING(    0x08, "Easy" )
	PORT_DIPSETTING(    0x0c, "Normal" )
	PORT_DIPSETTING(    0x04, "Hard" )
	PORT_DIPSETTING(    0x00, "Hardest" )
	PORT_DIPNAME( 0x10, 0x10, "Unknown", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "Off" )
	PORT_DIPSETTING(    0x10, "On" )
	PORT_DIPNAME( 0x20, 0x20, "Unused", IP_KEY_NONE )
	PORT_DIPSETTING(    0x20, "Off" )
	PORT_DIPSETTING(    0x00, "On" )
	PORT_DIPNAME( 0x40, 0x40, "Unused", IP_KEY_NONE )
	PORT_DIPSETTING(    0x40, "Off" )
	PORT_DIPSETTING(    0x00, "On" )
	PORT_DIPNAME( 0x80, 0x80, "Demo Sounds", IP_KEY_NONE )
	PORT_DIPSETTING(    0x80, "Off" )
	PORT_DIPSETTING(    0x00, "On" )
INPUT_PORTS_END

/**********************************************************************************/

static struct GfxLayout charlayout =
{
	8,8,	/* 8*8 chars */
	4096,
	4,		/* 4 bits per pixel  */
	{ 0x08000*8, 0x18000*8, 0x00000*8, 0x10000*8 },
	{ 0, 1, 2, 3, 4, 5, 6, 7 },
	{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8 },
	8*8	/* every char takes 8 consecutive bytes */
};

static struct GfxLayout spritelayout =
{
	16,16,
	4096*4,  /* A lotta sprites.. */
	4,
	{ 8, 0, 0x100000*8+8, 0x100000*8 },
	{ 32*8+0, 32*8+1, 32*8+2, 32*8+3, 32*8+4, 32*8+5, 32*8+6, 32*8+7,
		0, 1, 2, 3, 4, 5, 6, 7 },
	{ 0*16, 1*16, 2*16, 3*16, 4*16, 5*16, 6*16, 7*16,
			8*16, 9*16, 10*16, 11*16, 12*16, 13*16, 14*16, 15*16 },
	64*8
};

static struct GfxLayout tilelayout =
{
	16,16,
	4096,
	4,
	{ 0x40000*8+8, 0x40000*8, 8, 0,  },
	{ 32*8+0, 32*8+1, 32*8+2, 32*8+3, 32*8+4, 32*8+5, 32*8+6, 32*8+7,
		0, 1, 2, 3, 4, 5, 6, 7 },
	{ 0*16, 1*16, 2*16, 3*16, 4*16, 5*16, 6*16, 7*16,
			8*16, 9*16, 10*16, 11*16, 12*16, 13*16, 14*16, 15*16 },
	64*8
};

/* Areas 1536-2048 & 1024-1280 seem to be always empty */
static struct GfxDecodeInfo gfxdecodeinfo[] =
{
	{ 1, 0x000000, &charlayout,    0, 16 },	/* Characters 8x8 */
	{ 1, 0x020000, &tilelayout, 1280, 16 },	/* Tiles 16x16 */
	{ 1, 0x0a0000, &tilelayout,  512, 16 },	/* Tiles 16x16 */
	{ 1, 0x120000, &tilelayout,  256, 16 },	/* Tiles 16x16 */
	{ 1, 0x1a0000, &spritelayout,768, 16 },	/* Sprites 16x16 */
	{ -1 } /* end of array */
};

/**********************************************************************************/

static struct YM2203interface ym2203_interface =
{
	1,
	32220000/8,	/* Accurate */
	{ YM2203_VOL(29,29) },
	{ 0 },
	{ 0 },
	{ 0 },
	{ 0 }
};


static void sound_irq(void)
{
	cpu_cause_interrupt(1,H6280_INT_IRQ2);
}

static void sound_irq2(void)
{
	cpu_cause_interrupt(1,H6280_INT_IRQ2);
}

static struct YM2151interface ym2151_interface =
{
	1,
	32220000/8, /* Accurate */
	{ YM3012_VOL(40,OSD_PAN_LEFT,40,OSD_PAN_RIGHT) },
	{ sound_irq }
};

static struct YM2151interface ym2151_interface2 =
{
	1,
	32220000/8, /* Bootleg frequency - who knows.. */
	{ 50 },
	{ sound_irq2 }
};

static struct OKIM6295interface okim6295_interface =
{
	2,              /* 2 chips */
	8000,           /* 8000Hz frequency */
	{ 3,4 },        /* memory regions 3 & 4 */
	{ 40,10 }
};

/**********************************************************************************/

static struct MachineDriver cninja_machine_driver =
{
	/* basic machine hardware */
	{
		{
			CPU_M68000,
			12000000,
			0,
			cninja_readmem,cninja_writemem,0,0,
			m68_level5_irq,1
		},
		{
			CPU_H6280 | CPU_AUDIO_CPU,
			32220000/8,	/* Accurate */
			2,
			sound_readmem,sound_writemem,0,0,
			ignore_interrupt,0
		}
	},
	60, DEFAULT_REAL_60HZ_VBLANK_DURATION,
	1,	/* 1 CPU slice per frame - interleaving is forced when a sound command is written */
	0,

	/* video hardware */
	32*8, 32*8, { 0*8, 32*8-1, 1*8, 31*8-1 },

	gfxdecodeinfo,
	2048, 2048,
	0,

	VIDEO_TYPE_RASTER | VIDEO_MODIFIES_PALETTE,
	0,
	cninja_vh_start,
	cninja_vh_stop,
	cninja_vh_screenrefresh,

	/* sound hardware */
	0,0,0,0, /* Mono */
  	{
		{
			SOUND_YM2203,
			&ym2203_interface
		},
		{
			SOUND_YM2151_ALT,
			&ym2151_interface
		},
		{
			SOUND_OKIM6295,
			&okim6295_interface
		}
	}
};

static struct MachineDriver stoneage_machine_driver =
{
	/* basic machine hardware */
	{
		{
			CPU_M68000,
			12000000,
			0,
			cninja_readmem,cninja_writemem,0,0,
			m68_level5_irq,1
		},
		{
			CPU_Z80 | CPU_AUDIO_CPU,
			4000000,
			2,
			stoneage_s_readmem,stoneage_s_writemem,0,0,
			ignore_interrupt,0
		}
	},
	60, DEFAULT_REAL_60HZ_VBLANK_DURATION,
	1,	/* 1 CPU slice per frame - interleaving is forced when a sound command is written */
	0,

	/* video hardware */
	32*8, 32*8, { 0*8, 32*8-1, 1*8, 31*8-1 },

	gfxdecodeinfo,
	2048, 2048,
	0,

	VIDEO_TYPE_RASTER | VIDEO_MODIFIES_PALETTE,
	0,
	cninja_vh_start,
	cninja_vh_stop,
	cninja_vh_screenrefresh,

	/* sound hardware */
	0,0,0,0, /* Mono */
	{
		{
			SOUND_YM2151,
			&ym2151_interface2
		},
		{
			SOUND_OKIM6295,
			&okim6295_interface
		}
	}
};

/**********************************************************************************/

ROM_START( cninja_rom )
	ROM_REGION(0xc0000) /* 68000 code */
	ROM_LOAD_EVEN( "gn02rev3.bin", 0x00000, 0x20000, 0x39aea12a )
	ROM_LOAD_ODD ( "gn05rev2.bin", 0x00000, 0x20000, 0x0f4360ef )
	ROM_LOAD_EVEN( "gn01rev2.bin", 0x40000, 0x20000, 0xf740ef7e )
	ROM_LOAD_ODD ( "gn04rev2.bin", 0x40000, 0x20000, 0xc98fcb62 )
	ROM_LOAD_EVEN( "gn-00.rom",    0x80000, 0x20000, 0x0b110b16 )
	ROM_LOAD_ODD ( "gn-03.rom",    0x80000, 0x20000, 0x1e28e697 )

	ROM_REGION(0x3a0000) /* temporary space for graphics (disposed after conversion) */
	ROM_LOAD( "gl-08.rom",  0x00000,  0x10000,  0x33a2b400 )	/* chars */
	ROM_LOAD( "gl-09.rom",  0x10000,  0x10000,  0x5a2d4752 )

	ROM_LOAD( "mag-00.rom", 0x020000, 0x80000,  0xa8f05d33 ) /* tiles 1 */
	ROM_LOAD( "mag-01.rom", 0x0a0000, 0x80000,  0x5b399eed ) /* tiles 2 */
	ROM_LOAD( "mag-02.rom", 0x120000, 0x80000,  0xde89c69a ) /* tiles 3 */

	ROM_LOAD( "mag-03.rom", 0x1a0000, 0x80000,  0x2220eb9f ) /* sprites */
	ROM_LOAD( "mag-04.rom", 0x220000, 0x80000,  0x144b94cc )
	ROM_LOAD( "mag-05.rom", 0x2a0000, 0x80000,  0x56a53254 )
	ROM_LOAD( "mag-06.rom", 0x320000, 0x80000,  0x82d44749 )

	ROM_REGION(0x10000) /* Sound CPU */
	ROM_LOAD( "gl-07.rom",  0x00000,  0x10000,  0xca8bef96 )

	ROM_REGION(0x20000) /* Oki samples */
	ROM_LOAD( "gl-06.rom",  0x00000,  0x20000,  0xd92e519d )

	ROM_REGION(0x80000) /* Extra Oki samples */
	ROM_LOAD( "mag-07.rom", 0x00000,  0x80000,  0x08eb5264 )	/* banked */
ROM_END

ROM_START( cninja0_rom )
	ROM_REGION(0xc0000) /* 68000 code */
	ROM_LOAD_EVEN( "gn-02.rom", 0x00000, 0x20000, 0xccc59524 )
	ROM_LOAD_ODD ( "gn-05.rom", 0x00000, 0x20000, 0xa002cbe4 )
	ROM_LOAD_EVEN( "gn-01.rom", 0x40000, 0x20000, 0x18f0527c )
	ROM_LOAD_ODD ( "gn-04.rom", 0x40000, 0x20000, 0xea4b6d53 )
	ROM_LOAD_EVEN( "gn-00.rom", 0x80000, 0x20000, 0x0b110b16 )
	ROM_LOAD_ODD ( "gn-03.rom", 0x80000, 0x20000, 0x1e28e697 )

	ROM_REGION(0x3a0000) /* temporary space for graphics (disposed after conversion) */
	ROM_LOAD( "gl-08.rom",  0x00000,  0x10000,  0x33a2b400 )	/* chars */
	ROM_LOAD( "gl-09.rom",  0x10000,  0x10000,  0x5a2d4752 )

	ROM_LOAD( "mag-00.rom", 0x020000, 0x80000,  0xa8f05d33 ) /* tiles 1 */
	ROM_LOAD( "mag-01.rom", 0x0a0000, 0x80000,  0x5b399eed ) /* tiles 2 */
	ROM_LOAD( "mag-02.rom", 0x120000, 0x80000,  0xde89c69a ) /* tiles 3 */

	ROM_LOAD( "mag-03.rom", 0x1a0000, 0x80000,  0x2220eb9f ) /* sprites */
	ROM_LOAD( "mag-04.rom", 0x220000, 0x80000,  0x144b94cc )
	ROM_LOAD( "mag-05.rom", 0x2a0000, 0x80000,  0x56a53254 )
	ROM_LOAD( "mag-06.rom", 0x320000, 0x80000,  0x82d44749 )

	ROM_REGION(0x10000) /* Sound CPU */
	ROM_LOAD( "gl-07.rom",  0x00000,  0x10000,  0xca8bef96 )

	ROM_REGION(0x20000) /* Oki samples */
	ROM_LOAD( "gl-06.rom",  0x00000,  0x20000,  0xd92e519d )

	ROM_REGION(0x80000) /* Extra Oki samples */
	ROM_LOAD( "mag-07.rom", 0x00000,  0x80000,  0x08eb5264 )	/* banked */
ROM_END

ROM_START( cninjau_rom )
	ROM_REGION(0xc0000) /* 68000 code */
	ROM_LOAD_EVEN( "gm02-3.1k", 0x00000, 0x20000, 0xd931c3b1 )
	ROM_LOAD_ODD ( "gm05-2.3k", 0x00000, 0x20000, 0x7417d3fb )
	ROM_LOAD_EVEN( "gm01-2.1j", 0x40000, 0x20000, 0x72041f7e )
	ROM_LOAD_ODD ( "gm04-2.3j", 0x40000, 0x20000, 0x2104d005 )
	ROM_LOAD_EVEN( "gn-00.rom", 0x80000, 0x20000, 0x0b110b16 )
	ROM_LOAD_ODD ( "gn-03.rom", 0x80000, 0x20000, 0x1e28e697 )

	ROM_REGION(0x3a0000) /* temporary space for graphics (disposed after conversion) */
	ROM_LOAD( "gl-08.rom",  0x00000,  0x10000,  0x33a2b400 )	/* chars */
	ROM_LOAD( "gl-09.rom",  0x10000,  0x10000,  0x5a2d4752 )

	ROM_LOAD( "mag-00.rom", 0x020000, 0x80000,  0xa8f05d33 ) /* tiles 1 */
	ROM_LOAD( "mag-01.rom", 0x0a0000, 0x80000,  0x5b399eed ) /* tiles 2 */
	ROM_LOAD( "mag-02.rom", 0x120000, 0x80000,  0xde89c69a ) /* tiles 3 */

	ROM_LOAD( "mag-03.rom", 0x1a0000, 0x80000,  0x2220eb9f ) /* sprites */
	ROM_LOAD( "mag-04.rom", 0x220000, 0x80000,  0x144b94cc )
	ROM_LOAD( "mag-05.rom", 0x2a0000, 0x80000,  0x56a53254 )
	ROM_LOAD( "mag-06.rom", 0x320000, 0x80000,  0x82d44749 )

	ROM_REGION(0x10000) /* Sound CPU */
	ROM_LOAD( "gl-07.rom",  0x00000,  0x10000,  0xca8bef96 )

	ROM_REGION(0x20000) /* Oki samples */
	ROM_LOAD( "gl-06.rom",  0x00000,  0x20000,  0xd92e519d )

	ROM_REGION(0x80000) /* Extra Oki samples */
	ROM_LOAD( "mag-07.rom", 0x00000,  0x80000,  0x08eb5264 )	/* banked */
ROM_END

ROM_START( joemac_rom )
	ROM_REGION(0xc0000) /* 68000 code */
	ROM_LOAD_EVEN( "gl02-2.k1", 0x00000, 0x20000,  0x80da12e2 )
	ROM_LOAD_ODD ( "gl05-2.k3", 0x00000, 0x20000,  0xfe4dbbbb )
	ROM_LOAD_EVEN( "gl01-2.j1", 0x40000, 0x20000,  0x0b245307 )
	ROM_LOAD_ODD ( "gl04-2.j3", 0x40000, 0x20000,  0x1b331f61 )
	ROM_LOAD_EVEN( "gn-00.rom", 0x80000, 0x20000,  0x0b110b16 )
	ROM_LOAD_ODD ( "gn-03.rom", 0x80000, 0x20000,  0x1e28e697 )

	ROM_REGION(0x3a0000) /* temporary space for graphics (disposed after conversion) */
	ROM_LOAD( "gl-08.rom",  0x00000,  0x10000,  0x33a2b400 )	/* chars */
	ROM_LOAD( "gl-09.rom",  0x10000,  0x10000,  0x5a2d4752 )

	ROM_LOAD( "mag-00.rom", 0x020000, 0x80000,  0xa8f05d33 ) /* tiles 1 */
	ROM_LOAD( "mag-01.rom", 0x0a0000, 0x80000,  0x5b399eed ) /* tiles 2 */
	ROM_LOAD( "mag-02.rom", 0x120000, 0x80000,  0xde89c69a ) /* tiles 3 */

	ROM_LOAD( "mag-03.rom", 0x1a0000, 0x80000,  0x2220eb9f ) /* sprites */
	ROM_LOAD( "mag-04.rom", 0x220000, 0x80000,  0x144b94cc )
	ROM_LOAD( "mag-05.rom", 0x2a0000, 0x80000,  0x56a53254 )
	ROM_LOAD( "mag-06.rom", 0x320000, 0x80000,  0x82d44749 )

	ROM_REGION(0x10000) /* Sound CPU */
	ROM_LOAD( "gl-07.rom",  0x00000,  0x10000,  0xca8bef96 )

	ROM_REGION(0x20000) /* Oki samples */
	ROM_LOAD( "gl-06.rom",  0x00000,  0x20000,  0xd92e519d )

	ROM_REGION(0x80000) /* Extra Oki samples */
	ROM_LOAD( "mag-07.rom", 0x00000,  0x80000,  0x08eb5264 )	/* banked */
ROM_END

ROM_START( stoneage_rom )
	ROM_REGION(0xc0000) /* 68000 code */
	ROM_LOAD_EVEN( "sa_1_019.bin", 0x00000, 0x20000,  0x7fb8c44f )
	ROM_LOAD_ODD ( "sa_1_033.bin", 0x00000, 0x20000,  0x961c752b )
	ROM_LOAD_EVEN( "sa_1_018.bin", 0x40000, 0x20000,  0xa4043022 )
	ROM_LOAD_ODD ( "sa_1_032.bin", 0x40000, 0x20000,  0xf52a3286 )
	ROM_LOAD_EVEN( "sa_1_017.bin", 0x80000, 0x20000,  0x08d6397a )
	ROM_LOAD_ODD ( "sa_1_031.bin", 0x80000, 0x20000,  0x103079f5 )

	ROM_REGION(0x3a0000) /* temporary space for graphics (disposed after conversion) */
	ROM_LOAD( "gl-08.rom",  0x00000,  0x10000,  0x33a2b400 )	/* chars */
	ROM_LOAD( "gl-09.rom",  0x10000,  0x10000,  0x5a2d4752 )

	/* The bootleg graphics are stored in a different arrangement but
		seem to be the same as the original set */

	ROM_LOAD( "mag-00.rom", 0x020000, 0x80000,  0xa8f05d33 ) /* tiles 1 */
	ROM_LOAD( "mag-01.rom", 0x0a0000, 0x80000,  0x5b399eed ) /* tiles 2 */
	ROM_LOAD( "mag-02.rom", 0x120000, 0x80000,  0xde89c69a ) /* tiles 3 */

	ROM_LOAD( "mag-03.rom", 0x1a0000, 0x80000,  0x2220eb9f ) /* sprites */
	ROM_LOAD( "mag-04.rom", 0x220000, 0x80000,  0x144b94cc )
	ROM_LOAD( "mag-05.rom", 0x2a0000, 0x80000,  0x56a53254 )
	ROM_LOAD( "mag-06.rom", 0x320000, 0x80000,  0x82d44749 )

	ROM_REGION(0x10000) /* Sound CPU */
	ROM_LOAD( "sa_1_012.bin",  0x00000,  0x10000, 0x56058934 )

	ROM_REGION(0x40000) /* Oki samples */
	ROM_LOAD( "sa_1_069.bin",  0x00000,  0x40000, 0x2188f3ca )

	ROM_REGION(0x100) /* No extra Oki samples in the bootleg */

ROM_END

/**********************************************************************************/

static void cninja_patch(void)
{
	unsigned char *RAM = Machine->memory_region[Machine->drv->cpu[0].memory_region];
	int i;

	for (i=0; i<0x80000; i+=2) {
		int aword=READ_WORD(&RAM[i]);

		if (aword==0x66ff || aword==0x67ff) {
			int doublecheck=READ_WORD(&RAM[i-8]);

			/* Cmpi + btst controlling opcodes */
			if (doublecheck==0xc39 || doublecheck==0x839) {

				WRITE_WORD (&RAM[i],0x4E71);
		        WRITE_WORD (&RAM[i-2],0x4E71);
		        WRITE_WORD (&RAM[i-4],0x4E71);
		        WRITE_WORD (&RAM[i-6],0x4E71);
		        WRITE_WORD (&RAM[i-8],0x4E71);
			}
		}
	}

	/* Remove this bit later.. */
	RAM = Machine->memory_region[2];
	RAM[0x1d6]=0xea; /* NOP out CLI */
}

/**********************************************************************************/

static void custom_memory(void)
{
	install_mem_write_handler(0, 0x1bc0a8, 0x1bc0a9, cninja_sound_w);
}

static void custom_memory2(void)
{
	install_mem_write_handler(0, 0x1bc0a8, 0x1bc0a9, stoneage_sound_w);
}

/**********************************************************************************/

struct GameDriver cninja_driver =
{
	__FILE__,
	0,
	"cninja",
	"Caveman Ninja (World revision 3)",
	"1991",
	"Data East Corporation",
	"Bryan McPhail",
	0,
	&cninja_machine_driver,
	custom_memory,

	cninja_rom,
	cninja_patch, 0,
	0,
	0,	/* sound_prom */

	input_ports,

	0, 0, 0,   /* colors, palette, colortable */
	ORIENTATION_DEFAULT,
	0, 0
};

struct GameDriver cninja0_driver =
{
	__FILE__,
	&cninja_driver,
	"cninja0",
	"Caveman Ninja (World revision 0)",
	"1991",
	"Data East Corporation",
	"Bryan McPhail",
	0,
	&cninja_machine_driver,
	custom_memory,

	cninja0_rom,
	cninja_patch, 0,
	0,
	0,	/* sound_prom */

	input_ports,

	0, 0, 0,   /* colors, palette, colortable */
	ORIENTATION_DEFAULT,
	0, 0
};


struct GameDriver cninjau_driver =
{
	__FILE__,
	&cninja_driver,
	"cninjau",
	"Caveman Ninja (US)",
	"1991",
	"Data East Corporation",
	"Bryan McPhail",
	0,
	&cninja_machine_driver,
	custom_memory,

	cninjau_rom,
	cninja_patch, 0,
	0,
	0,	/* sound_prom */

	cninjau_input_ports,

	0, 0, 0,   /* colors, palette, colortable */
	ORIENTATION_DEFAULT,
	0, 0
};

struct GameDriver joemac_driver =
{
	__FILE__,
	&cninja_driver,
	"joemac",
	"Joe & Mac (Japan)",
	"1991",
	"Data East Corporation",
	"Bryan McPhail",
	0,
	&cninja_machine_driver,
	custom_memory,

	joemac_rom,
	cninja_patch, 0,
	0,
	0,	/* sound_prom */

	input_ports,

	0, 0, 0,   /* colors, palette, colortable */
	ORIENTATION_DEFAULT,
	0, 0
};

struct GameDriver stoneage_driver =
{
	__FILE__,
	&cninja_driver,
	"stoneage",
	"Stoneage",
	"1991",
	"bootleg",
	"Bryan McPhail",
	0,
	&stoneage_machine_driver,
	custom_memory2,

	stoneage_rom,
	0, 0,
	0,
	0,	/* sound_prom */

	input_ports,

	0, 0, 0,   /* colors, palette, colortable */
	ORIENTATION_DEFAULT,
	0, 0
};
